/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/

/*
    socketmode.h

    Definition of routines to support NPort serial port control.

    2007-04-30	Kevin Chu
		new release
    2008-05-30	Albert Yu
		add IPv6 support
*/

#ifndef	_SOCKETMODE_H
#define	_SOCKETMODE_H

#ifdef	WIN32
	#include	<winsock2.h>
	#include	<ws2tcpip.h>
	typedef union sockaddr_union{
	        struct sockaddr_in  AddressIn;
#ifdef IPV6_ON
	        struct sockaddr_in6 AddressIn6;
#endif
	} Sockaddr;	/* we don't use sockaddr_gen defined in ws2tcpip.h because it may use sockaddr_in6_old */
	#define Sin_family	AddressIn.sin_family
	#define Sin_port	AddressIn.sin_port
	#define Sin_addr	AddressIn.sin_addr
	#define Sin_zero	AddressIn.sa4.sin4_zero
	#define Sin6_family	AddressIn6.sin6_family
	#define Sin6_port	AddressIn6.sin6_port
	#define Sin6_flowinfo	AddressIn6.sin6_flowinfo
	#define Sin6_addr	AddressIn6.sin6_addr
	#define Sin6_scope_id	AddressIn6.sin6_scope_id
#else
	#define	ioctlsocket	ioctl
	#define closesocket close
	typedef int		SOCKET;
	typedef union sockaddr_union{
	        struct sockaddr_in  AddressIn;
#ifdef IPV6_ON
	        struct sockaddr_in6 AddressIn6;
#endif
	} Sockaddr;	
	#define Sin_family	AddressIn.sin_family
	#define Sin_port	AddressIn.sin_port
	#define Sin_addr	AddressIn.sin_addr
	#define Sin_zero	AddressIn.sa4.sin4_zero
	#define Sin6_family	AddressIn6.sin6_family
	#define Sin6_port	AddressIn6.sin6_port
	#define Sin6_flowinfo	AddressIn6.sin6_flowinfo
	#define Sin6_addr	AddressIn6.sin6_addr
	#define Sin6_scope_id	AddressIn6.sin6_scope_id
#endif

#define DFD_TABLE_SIZE	128						/* Max. async ports to be control */

#define D_START_DATAPORT	4001				/* 4001: TCP port no for 1st data port  */
                                    	/* 4002: TCP port no for 2nd data port  */
                                    	/* 4003: TCP port no for 3rd data port  */
                                    	/* 4004: TCP port no for 4th data port  */
                                    	/* ....				      */
                                    	/* 4016: TCP port no for 16th data port */

#define D_START_CMDPORT 	966					/* 966: TCP port no for 1st cmd port    */
                                    	/* 967: TCP port no for 2nd cmd port    */
                                    	/* 968: TCP port no for 3rd cmd port    */
                                    	/* ....				      */
                                    	/* 981: TCP port no for 16th cmd port   */

#define DCMD_IOCTL				16          /* IOCTL command	       */
#define DCMD_FLOWCTRL			17          /* Flow control command        */
#define DCMD_LINECTRL			18          /* Line status control command */
#define DCMD_LSTATUS			19          /* Line status read command    */
#define DCMD_FLUSH				20          /* Buffer flush command        */
#define DCMD_IQUEUE				21          /* Input queue command	       */
#define DCMD_OQUEUE				22          /* Output queue command        */
#define DCMD_BAUDRATE			23          /* Set port's baud rate command */
#define DCMD_BREAK				35          /* Send break in mini-seconds  */
#define	DCMD_BREAKCNT			55          /* Get break count command */
#define DCMD_DSTATUS			56          /* Data status read command */

#define DCMD_IPV6_INIT			99          /* Init IPv6 command port */

/* parameters for D_COMMAND_IOCTL ioctl command */
#define D_IOCTL_B300			0           /* IOCTL : baud rate = 300 bps	 */
#define D_IOCTL_B600			1           /* IOCTL : baud rate = 600 bps	 */
#define D_IOCTL_B1200			2           /* IOCTL : baud rate = 1200 bps  */
#define D_IOCTL_B2400			3           /* IOCTL : baud rate = 2400 bps  */
#define D_IOCTL_B4800			4           /* IOCTL : baud rate = 4800 bps  */
#define D_IOCTL_B7200			5           /* IOCTL : baud rate = 7200 bps  */
#define D_IOCTL_B9600			6           /* IOCTL : baud rate = 9600 bps  */
#define D_IOCTL_B19200		7           /* IOCTL : baud rate = 19200 bps */
#define D_IOCTL_B38400		8           /* IOCTL : baud rate = 38400 bps */
#define D_IOCTL_B57600		9           /* IOCTL : baud rate = 57600 bps */
#define D_IOCTL_B115200 	10          /* IOCTL : baud rate = 115200 bps */
#define D_IOCTL_B230400 	11          /* IOCTL : baud rate = 230400 bps */
#define D_IOCTL_B460800 	12          /* IOCTL : baud rate = 460800 bps */
#define D_IOCTL_B921600 	13          /* IOCTL : baud rate = 921600 bps */
#define D_IOCTL_150	    	14          /* IOCTL : baud rate = 150 bps */
#define D_IOCTL_134	    	15          /* IOCTL : baud rate = 134.5 bps */
#define D_IOCTL_110	    	16          /* IOCTL : baud rate = 110 bps */
#define D_IOCTL_75	    	17          /* IOCTL : baud rate = 75 bps */
#define D_IOCTL_50	    	18          /* IOCTL : baud rate = 50 bps */

#define D_IOCTL_BIT8			3           /* IOCTL : 8 data bits */
#define D_IOCTL_BIT7			2           /* IOCTL : 7 data bits */
#define D_IOCTL_BIT6			1           /* IOCTL : 6 data bits */
#define D_IOCTL_BIT5			0           /* IOCTL : 5 data bits */

#define D_IOCTL_STOP1			0           /* IOCTL : 1 stop bit  */
#define D_IOCTL_STOP2			4           /* IOCTL : 2 stop bits */

#define D_IOCTL_NONE			0           /* IOCTL : none parity */
#define D_IOCTL_EVEN			8           /* IOCTL : even parity */
#define D_IOCTL_ODD				16          /* IOCTL : odd parity  */
#define D_IOCTL_MARK			24          /* IOCTL : mark parity	*/
#define D_IOCTL_SPACE			32          /* IOCTL : space parity  */


/* parameters for D_COMMAND_FLOWCTRL flow control command */
#define D_FCTRL_CTS_YES 	1           /* yes, do CTS H/W flow control      */
#define D_FCTRL_CTS_NO		0           /* no, don't do CTS H/W flow control */
#define D_FCTRL_RTS_YES 	1           /* yes, do RTS H/W flow control      */
#define D_FCTRL_RTS_NO		0           /* no, don't do RTS H/W flow control */
#define D_FCTRL_TXXON_YES	1           /* yes, Tx XON/XOFF S/W flow control */
#define D_FCTRL_TXXON_NO	0           /* no Tx XON/XOFF S/W flow control   */
#define D_FCTRL_RXXON_YES	1           /* yes, Rx XON/XOFF S/W flow control */
#define D_FCTRL_RXXON_NO	0           /* no Rx XON/XOFF S/W flow control   */

/* parameters for D_COMMAND_LINECTRL line control command	*/
#define D_LCTRL_DTR_ON		1           /* set DTR on		*/
#define D_LCTRL_DTR_OFF 	0           /* set DTR off		*/
#define D_LCTRL_RTS_ON		1           /* set RTS on		*/
#define D_LCTRL_RTS_OFF 	0           /* set RTS off		*/

/* parameters for D_COMMAND_FLUSH flush buffer command		*/
#define D_FLUSH_RXBUFFER	0           /* flush Rx buffer	*/
#define D_FLUSH_TXBUFFER	1           /* flush Tx buffer	*/
#define D_FLUSH_ALLBUFFER	2           /* flush Rx & Tx buffer */

/* parameters for D_COMMAND_LSTATUS line status command */
#define D_LSTATUS_DSR_ON	1           /* line status : DSR on 	*/
#define D_LSTATUS_DSR_OFF	0           /* line status : DSR off	*/
#define D_LSTATUS_CTS_ON	1           /* line status : CTS on 	*/
#define D_LSTATUS_CTS_OFF	0           /* line status : CTS off	*/
#define D_LSTATUS_DCD_ON	1           /* line status : DCD on 	*/
#define D_LSTATUS_DCD_OFF	0           /* line status : DCD off	*/

/* return code */
#define SSIO_OK 								0 		/* OK */
#define SSIO_BADPORT						-1 		/* No such port or port not opened */ 
#define SSIO_BADPARM						-2 		/* Bad parameter */
#define SSIO_SOCK_INIT_FAIL 		-200  /* Socket initialize error */ 
#define SSIO_SOCK_MAX_ERR				-201  /* Max connection reached */
#define SSIO_CONNECT_FAIL 			-202  /* Connect to NPort fail */ 
#define SSIO_SOCK_ERR 					-203  /* Communication with NPort error */
#define	SSIO_NO_CMDPORT					-204	/* No such command port */
#define SSIO_SSL_INIT_FAIL  		-300	/* SSL initialize error */
#define SSIO_SSL_HANDSHAKE_ERR	-301  /* SSL handshake fail */
#define	SSIO_NO_SSL_KEY					-302	/* No such SSL key */


/*
 *	Global functions:
 */
int	ssio_init(void);
int	ssio_open(char *server_ip, int cmd_port, int dataport, int ssl_enable);
int	ssio_close(SOCKET fd);
int	ssio_ioctl(SOCKET fd, int baud, int mode);
int	ssio_baud(SOCKET fd, long baud);
int	ssio_flowctrl(SOCKET fd, int mode);
int	ssio_lctrl(SOCKET fd, int mode);
int	ssio_lstatus(SOCKET fd);
int	ssio_flush(SOCKET fd, int func);
int	ssio_write(SOCKET fd, char *buf, int len);
int	ssio_read(SOCKET fd, char *buf, int len);
int	ssio_break(SOCKET fd, int time);
int	ssio_oqueue(SOCKET fd);
int	ssio_iqueue(SOCKET fd);
int	ssio_breakcnt(SOCKET fd);
int	ssio_data_status(SOCKET fd);

#endif

